@@ -1,4 +1,4 @@
-"""Parasect."""
+"""Parasect package."""
 from ._helpers import Formats
 from .build_lib import build_helper as build
 from .compare_lib import compare_helper as compare
